#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  /// <summary>
  /// Represents a time series prognosis data analysis solution
  /// </summary>
  [StorableClass]
  public abstract class TimeSeriesPrognosisSolution : TimeSeriesPrognosisSolutionBase {

    [StorableConstructor]
    protected TimeSeriesPrognosisSolution(bool deserializing)
      : base(deserializing) {
    }
    protected TimeSeriesPrognosisSolution(TimeSeriesPrognosisSolution original, Cloner cloner)
      : base(original, cloner) {
    }
    protected TimeSeriesPrognosisSolution(ITimeSeriesPrognosisModel model, ITimeSeriesPrognosisProblemData problemData)
      : base(model, problemData) {
    }

    protected override void RecalculateResults() {
      CalculateResults();
    }

    public override IEnumerable<IEnumerable<double>> PrognosedTrainingValues {
      get {
        return GetPrognosedValues(ProblemData.TrainingIndizes.Take(1),
          ProblemData.TrainingPartition.End - ProblemData.TrainingPartition.Start)
          .First();
      }
    }
    public override IEnumerable<IEnumerable<double>> PrognosedTestValues {
      get {
        return GetPrognosedValues(ProblemData.TestIndizes.Take(1),
          ProblemData.TestPartition.End - ProblemData.TestPartition.Start)
          .First();
      }
    }
    public override IEnumerable<IEnumerable<IEnumerable<double>>> GetPrognosedValues(IEnumerable<int> rows, int horizon) {
      return Model.GetPrognosedValues(ProblemData.Dataset, rows, horizon);
    }

    protected override void OnProblemDataChanged() {
      base.OnProblemDataChanged();
    }

    protected override void OnModelChanged() {
      base.OnModelChanged();
    }
  }
}
